//****************************************
//****************************************
//****************************************

translations["_project_point_distance"] = {"English": "Enter distance (in meters)", "Czech": "Zadej vzdálenost (v metrech)"};
translations["_project_point_azimuth"] = {"English": "Enter azimuth (in degrees)", "Czech": "Zadej azimut (ve stupních)"};
translations["_project_point_delete"] = {"English": "Delete", "Czech": "Smazat"};
translations["_project_point_set_distance"] = {"English": "Set distance...", "Czech": "Nastav vzdálenost..."};
translations["_project_point_set_azimuth"] = {"English": "Set azimuth...", "Czech": "Nastav azimut..."};
//translations["_project_point_delete_ask"] = {"English": "Do you really want to delete the points?", "Czech": "Opravdu chcete body smazat?"};

translations["_project_point_info_start"] = {"English": "Start", "Czech": "Začátek"};
translations["_project_point_info_end"] = {"English": "End", "Czech": "Konec"};
translations["_project_point_info_distance"] = {"English": "Distance", "Czech": "Vzdálenost"};
translations["_project_point_info_azimuth"] = {"English": "Azimuth", "Czech": "Azimut"};

function projectPoint(coords, distance, azimuth)
{
  this.distance = distance;
  this.azimuth = azimuth;
  this.pointStart = coords;
  this.pointEnd = this.pointStart.projectPoint(azimuth, distance);
  var that = this;
  
  // create layers for project point - if not already created  
  if (!window.projectPointLayer)
  {
    window.projectPointLayer = new SMap.Layer.Marker();
    m.addLayer(window.projectPointLayer).enable();

    window.projectPointGeomLayer = new SMap.Layer.Geometry();
    m.addLayer(window.projectPointGeomLayer).enable();
  }        
  
  // create line between points
  function drawLine(pointStart, pointEnd, old)
  {
    var options = {color: "red", width: 3, outlineWidth: 0, opacity: 0.85};
    var line = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, [pointStart, pointEnd], options);
    window.projectPointGeomLayer.addGeometry(line);
    
    drawArrow();
    return line;  
  }
  
  // create direction arrow
  function drawArrow()
  {
    if (that.arrowGeom)
      window.projectPointGeomLayer.removeGeometry(that.arrowGeom);

    var arrowPoint = that.pointStart.projectPoint(that.azimuth, that.distance / 2);
    var arrowAngle = 20;
    var arrowLength = (m.getScale() / 10000) * 20; 
    var arrowPt2 = arrowPoint.projectPoint(that.azimuth + 180 + arrowAngle, arrowLength);
    var arrowPt3 = arrowPoint.projectPoint(that.azimuth + 180 - arrowAngle, arrowLength);

    var options = {color: "red", width: 6, outlineWidth: 0, outlineColor: "red", opacity: 1}; 
    that.arrowGeom = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, [arrowPoint, arrowPt2, arrowPt3], options);
    window.projectPointGeomLayer.addGeometry(that.arrowGeom);      
  }
  
  this.onMapRedraw = m.getSignals().addListener(window, "map-redraw", function(e)
  {
    drawArrow();
  });  

  // create project point start
  this.markerStartDiv = JAK.mel("div", {className: "projectPointStart"});
  this.markerStart = new SMap.Marker(this.pointStart, null, {url: this.markerStartDiv, anchor: {left: 4, top: 4}});
  window.projectPointLayer.addMarker(this.markerStart);
  this.markerStart.decorate(SMap.Marker.Feature.Draggable);
  
  // create projected point
  this.markerEndDiv = JAK.mel("div", {className: "projectPointEnd"});
  this.markerEnd = new SMap.Marker(this.pointEnd, null, {url: this.markerEndDiv, anchor: {left: 4, top: 4}});
  window.projectPointLayer.addMarker(this.markerEnd);
  this.markerEnd.decorate(SMap.Marker.Feature.Draggable);
  
  // create bubble
  this.markerStart.decorate(SMap.Marker.Feature.Bubble, new SMap.Bubble({minWidth: 200}));
  this.markerEnd.decorate(SMap.Marker.Feature.Bubble, new SMap.Bubble({minWidth: 200}));
  
  // create card
  this.card = new SMap.Card(220);
  this.markerStart.decorate(SMap.Marker.Feature.Card, this.card);
  this.markerEnd.decorate(SMap.Marker.Feature.Card, this.card);
  
  // set bubble and card
  function setProjectPointInfo(/*pointStart, pointEnd, distance, azimuth*/)
  {
    var html = tr("_project_point_info_start") + ": " + that.pointStart.format("WGS84", "min")
      + "<br>" + tr("_project_point_info_end") + ": " + that.pointEnd.format("WGS84", "min") 
      + "<br>" + tr("_project_point_info_distance") + ": " + parseFloat(that.distance).toFixed(1) + " m"
      + "<br>" + tr("_project_point_info_azimuth") + ": " + parseFloat(that.azimuth).toFixed(3) + "°";
    that.markerEnd.bubble.setHTML(html);
    that.markerStart.bubble.setHTML(html);
    that.card.getBody().innerHTML = html;  
  }      
  
  setProjectPointInfo(this.pointStart, this.pointEnd, this.distance, this.azimuth);
  
  // add markers to layer
  window.projectPointLayer.addMarker(this.markerStart);
  window.projectPointLayer.addMarker(this.markerEnd);
  
  // create line
  this.markerStart.projectPointLine = drawLine(this.pointStart, this.pointEnd);
  
  // drag start or end points
  function redraw()
  {
    window.projectPointGeomLayer.removeGeometry(that.markerStart.projectPointLine);
    that.markerStart.projectPointLine = drawLine(that.pointStart, that.pointEnd);
    setProjectPointInfo(that.pointStart, that.pointEnd);  
  }
  
  this.markerDragHandler = m.getSignals().addListener(window, "marker-drag-move", function(e, target)
  {
    var marker = e.target;
  
    if (marker !== that.markerStart && marker !== that.markerEnd)
      return;

    if (marker === that.markerStart)
      that.pointStart = marker.getCoords();
    else
      that.pointEnd = marker.getCoords();

    // CTRL - move both points and leave distance/azimuth intact
    if (contains_class(m.getContainer(), "key_down_ctrl"))
    {
      if (marker === that.markerStart)
      {
      
        that.pointEnd = that.pointStart.projectPoint(that.azimuth, that.distance)
        that.markerEnd.setCoords(that.pointEnd);
      }
      else
      {
      
        that.pointStart = that.pointEnd.projectPoint(that.azimuth + 180, that.distance)
        that.markerStart.setCoords(that.pointStart);
      }    
    }

    // else - just move the point and recalculate distance and azimuth
    else
    {
   
  
      that.distance = that.pointStart.distance(that.pointEnd);
      that.azimuth = that.pointStart.azimuth(that.pointEnd);
    }

    redraw();
    marker.bubble.show();
  });
  
  // *** context menu ***
  var cMenu = "" +  
    "<ul class=\"context_menu\" style=\"display: none\">\n" +
    "  <li class=\"project_cmenu_delete\">" + tr("_project_point_delete") + "</li>\n" +
    "  <li class=\"project_cmenu_set_distance\">" + tr("_project_point_set_distance") + "</li>\n" +
    "  <li class=\"project_cmenu_set_azimuth\">" + tr("_project_point_set_azimuth") + "</li>\n" +
    "  <li class=\"project_cmenu_project_point\">" + tr("Project point...") + "</li>\n" +
    "  <li class=\"project_cmenu_draw_circle\">" + tr("Draw a circle...") + "</li>\n" +
    "</ul>";

  var cMenuDiv = document.createElement("div");
  cMenuDiv.innerHTML = cMenu;
  cMenu = cMenuDiv.firstChild;
  document.documentElement.appendChild(cMenu);

  function contextMenuHandler(e, target, menu, options) 
  {
    var coords = options.point.getCoords();

    function destroy()
    {
      window.projectPointGeomLayer.removeGeometry(that.arrowGeom);
      window.projectPointGeomLayer.removeGeometry(that.markerStart.projectPointLine);
      m.getSignals().removeListener(that.onMapRedraw);
      m.getSignals().removeListener(that.markerDragHandler);
      contextMenu.removeContextMenu(that.cMenuStartIndex);
      contextMenu.removeContextMenu(that.cMenuEndIndex);
      window.projectPointLayer.removeMarker(that.markerStart);
      window.projectPointLayer.removeMarker(that.markerEnd);
    }
  
    // delete
    if (contains_class(target, "project_cmenu_delete"))
    {
      destroy();
    }
    
    // set distance
    else if (contains_class(target, "project_cmenu_set_distance"))
    {
      ui("copybox", {text: that.distance, width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_distance"),
        getText: function(text)
        {
          var distance = parseFloat(text);
          
          if (!distance)
            return;
          
          that.distance = distance;
          that.pointEnd = that.pointStart.projectPoint(that.azimuth, that.distance);
          that.markerEnd.setCoords(that.pointEnd);  
          redraw();
        }
      });
    }
    
    // set azimuth
    else if (contains_class(target, "project_cmenu_set_azimuth"))
    {
      ui("copybox", {text: that.azimuth, width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_azimuth"),
        getText: function(text)
        {
          if (!text.length)
            return;
  
          var azimuth = parseFloat(text);
          
          if (isNaN(azimuth))
            return;
            
          that.azimuth = azimuth;
          that.pointEnd = that.pointStart.projectPoint(that.azimuth, that.distance);
          that.markerEnd.setCoords(that.pointEnd);
          redraw();
        }
      });
    }
    
    // project point (recursive)
    else if (contains_class(target, "project_cmenu_project_point"))
      projectPointOnMap(coords);
      
    // draw a circle
    else if (contains_class(target, "project_cmenu_draw_circle"))
      createCircleOnMap(coords);      
  }

  this.cMenuStartIndex = contextMenu.addContextMenu(this.markerStartDiv, cMenu, function (e, target, menu, options) {}, contextMenuHandler, {point: this.markerStart});
  this.cMenuEndIndex = contextMenu.addContextMenu(this.markerEndDiv, cMenu, function (e, target, menu, options) {}, contextMenuHandler, {point: this.markerEnd});
}

function projectPointOnMap(coords)
{
  ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_distance"),
    getText: function(text)
    {
      var distance = parseFloat(text);
      
      if (!distance)
        return;

      ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_project_point_azimuth"),
        getText: function(text)
        {
          if (!text.length)
            return;
  
          var azimuth = parseFloat(text);
          
          if (isNaN(azimuth))
            return;
          
          new projectPoint(coords, distance, azimuth);
        }
      });
    }
  });  
}

//****************************************
//****************************************
//****************************************

translations["_draw_circle_radius_ask"] = {"English": "Enter circle radius (in meters)", "Czech": "Zadej poloměr (v metrech)"};
translations["_draw_circle_radius"] = {"English": "Radius", "Czech": "Poloměr"};
//translations["_draw_circle_center_ask"] = {"English": "Enter circle center coordinates", "Czech": "Zadej souřadnice středu"};
translations["_draw_circle_center"] = {"English": "Center", "Czech": "Střed"};
translations["_draw_circle_delete"] = {"English": "Delete", "Czech": "Smazat"};
//translations["_draw_circle_set_center"] = {"English": "Set center...", "Czech": "Nastav střed..."};
translations["_draw_circle_set_radius"] = {"English": "Set radius...", "Czech": "Nastav poloměr..."};
//translations["_draw_circle_delete_ask"] = {"English": "Do you really want to delete the circle?", "Czech": "Opravdu chcete kruh smazat?"};

function createCircle(coords, radius)
{
  this.radius = radius;
  this.coords = coords;
  var that = this;

  // create layer (if not already exists)
  if (!window.drawCircleLayer)
  {
    window.drawCircleCenterLayer = new SMap.Layer.Marker();
    m.addLayer(window.drawCircleCenterLayer).enable();  
  
    window.drawCircleLayer = new SMap.Layer.Geometry("drawCircleLayer");
    m.addLayer(window.drawCircleLayer).enable();
  }      
  
  // create circle
  function drawCircle(center, radius)
  {
    var point_on_circle = center.projectPoint(0, radius);
    var circle = new SMap.Geometry(SMap.GEOMETRY_CIRCLE, null, [center, point_on_circle], {color: "#00f", opacity: 0.3, outlineWidth: 0});
    window.drawCircleLayer.addGeometry(circle);
    return circle;        
  }

  // create circle center
  this.markerDiv = JAK.mel("div", {className: "circleCenter"});
  this.marker = new SMap.Marker(this.coords, null, {url: this.markerDiv, anchor: {left: 4, top: 4}});
  this.marker.decorate(SMap.Marker.Feature.Draggable);
  
  // create bubble
  var bubble = new SMap.Bubble({minWidth: 200});
  this.marker.decorate(SMap.Marker.Feature.Bubble, bubble);
  
  // create card
  this.card = new SMap.Card(200);
  this.marker.decorate(SMap.Marker.Feature.Card, this.card);
  
  // set bubble and card
  function setCircleInfo(coords, radius)
  {
    var html = tr("_draw_circle_center") + ": " + coords.format("WGS84", "min") 
      + "<br>" + tr("_draw_circle_radius") + ": " + parseFloat(radius).toFixed(1) + " m";
    that.marker.bubble.setHTML(html);
    that.card.getBody().innerHTML = html;  
  }      
  
  setCircleInfo(this.coords, this.radius);
  
  // add marker to layer
  window.drawCircleCenterLayer.addMarker(this.marker);
  
  // create circle
  this.marker.drawCircle = drawCircle(this.coords, this.radius);
  
  // move circle on drag
  this.markerDragHandler = m.getSignals().addListener(window, "marker-drag-move", function(e, target)
  {
    var marker = e.target;
  
    if (marker !== that.marker)
      return;
  
    var dragCoords = marker.getCoords();
    window.drawCircleLayer.removeGeometry(marker.drawCircle);            
    
    // CTRL -> resize
    if (contains_class(m.getContainer(), "key_down_ctrl"))
      that.radius = that.coords.distance(dragCoords);
  
    // else -> move
    else
      that.coords = dragCoords;
    
    marker.drawCircle = drawCircle(that.coords, that.radius);
    setCircleInfo(that.coords, that.radius);
    marker.bubble.show();
  });
  
  this.markerDragStopHandler = m.getSignals().addListener(window, "marker-drag-stop", function(e, target)
  {
    var marker = e.target;
  
    if (marker !== that.marker)
      return;
  
    // set center marker
    marker.setCoords(that.coords);
    
    // update coords / radius
    setCircleInfo(that.coords, that.radius);
  });
  
  // *** context menu ***
  var cMenu = "" +  
    "<ul class=\"context_menu\" style=\"display: none\">\n" +
    "  <li class=\"circle_cmenu_delete\">" + tr("_draw_circle_delete") + "</li>\n" +
    //"  <li class=\"circle_cmenu_set_center\">" + tr("_draw_circle_set_center") + "</li>\n" +
    "  <li class=\"circle_cmenu_set_radius\">" + tr("_draw_circle_set_radius") + "</li>\n" +
    "  <li class=\"circle_cmenu_project_point\">" + tr("Project point...") + "</li>\n" +
    "  <li class=\"circle_cmenu_draw_circle\">" + tr("Draw a circle...") + "</li>\n" +    
    "</ul>";

  var cMenuDiv = document.createElement("div");
  cMenuDiv.innerHTML = cMenu;
  cMenu = cMenuDiv.firstChild;
  document.documentElement.appendChild(cMenu);

  this.cMenuIndex = contextMenu.addContextMenu(this.markerDiv, cMenu, 
  function (e, target, menu, options) {},
  function (e, target, menu, options) 
  {
    function destroy()
    {
      window.drawCircleLayer.removeGeometry(that.marker.drawCircle);
      m.getSignals().removeListener(that.markerDragHandler);
      m.getSignals().removeListener(that.markerDragStopHandler);
      contextMenu.removeContextMenu(that.cMenuIndex);
      window.drawCircleCenterLayer.removeMarker(that.marker);    
    }
  
    if (contains_class(target, "circle_cmenu_delete"))
      destroy();

    else if (contains_class(target, "circle_cmenu_set_center"))
    {
//       ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_draw_circle_center_ask"),
//         getText: function(text)
//         {
//           var center = parseFloat(text); /* ? */
//     
//           if (center)
//           {
//             window.drawCircleLayer.removeGeometry(that.marker.drawCircle);            
//             that.center = center;
//             that.marker.drawCircle = drawCircle(that.coords, that.radius);
//           }
//         }
//       });
    }
    else if (contains_class(target, "circle_cmenu_set_radius"))
    {
      ui("copybox", {text: that.radius, width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_draw_circle_radius_ask"),
        getText: function(text)
        {
          var radius = parseFloat(text);
    
          if (radius)
          {
            window.drawCircleLayer.removeGeometry(that.marker.drawCircle);            
            that.radius = radius;
            that.marker.drawCircle = drawCircle(that.coords, that.radius);
          }
        }
      });
    }
    
    // project point (recursive)
    else if (contains_class(target, "circle_cmenu_project_point"))
      projectPointOnMap(that.coords);
      
    // draw a circle
    else if (contains_class(target, "circle_cmenu_draw_circle"))
      createCircleOnMap(that.coords);    
  }); 
}

function createCircleOnMap(coords)
{
  ui("copybox", {text: "", width: 200, height: 50, select: true, exitOnEnter: true, closeButtonText: tr("_draw_circle_radius_ask"),
    getText: function(text)
    {
      var radius = parseFloat(text);

      if (radius)
        new createCircle(coords, radius);
    }
  });
}

