// encapsulate original function
function encapsulate (object, variable, fn)
{
  object[variable + "Orig"] = object[variable];
  object[variable] = fn;
}

// ************
// function setCenterZoom has no business calling setProjection
// (fixes reseting projection to UTM33 where zoom 19 was set)
// - our method may have issues, so replace the calling function instead
encapsulate(SMap.prototype, "setProjection", function(projection)
{
  if(arguments.callee.caller != this.setCenterZoom)
    this.setProjectionOrig(projection);
  // zoom may not be updated properly after leaving zoom 19, so fix it
  else
    this.redraw();
});

// setProjection without redraw
 SMap.prototype.setProjection2 = function(projection, redraw)
 {
 	if (this._projection) { this._projection.setOwner(null); }
 	this._projection = projection;
 	this._projection.setOwner(this);
  
//   if (redraw)
//  	  this._redraw(true); /* po zmene orientace je nutne plne prekresleni */

 	return this;
}

// ************
// get current layer + automatically switch projection + let us know before turning on the layer
encapsulate(SMap.Control.Layer.prototype, "_itemClick", function(e, elm)
{
  for (var id in this._layers) 
  {
  	if (this._layers[id] == elm)
    {
      var map = this.getMap();

      // set nearest zoom level
      map._zoom = map.getNearestZoom(id);

      // set new ID
      map.CurrentMapId = id;
      
      // set new projection (needed for correct copyright get scale - otherwise would use old projection)
      var conf = SMap.CONFIG[id];
      map.setProjection2((conf.options && conf.options.defaultProjection /* override default projection and reproject tiles to it */)
       || /* projection in CONFIG settings */ conf.projection || /* default projection */ SMap.projUTM33);
    }
  }

  this._itemClickOrig(e, elm);
});

encapsulate(SMap.Layer.prototype, "enable", function()
{
  if (SMap.CONFIG[this.getId()] && SMap.CONFIG[this.getId()].preview)
  {
    // no ID set yet? then set it (will be updated in _itemClick)
    this.getMap().CurrentMapId = this.getMap().CurrentMapId || this.getId();

    // set zoom range for the new map
    var zoomRange = SMap.CONFIG[this.getId()].zoom;
    
    if (zoomRange && zoomRange.length)  
      this.getMap().setZoomRange.apply(this.getMap(), zoomRange);
   
    // change layer
    if (this.getId() == this.getMap().CurrentMapId)
    {
      // set new projection again (in case this is our first run)
      var conf = SMap.CONFIG[this.getId()];
      this.getMap().setProjection2((conf.options && conf.options.defaultProjection /* override default projection and reproject tiles to it */)
       || /* projection in CONFIG settings */ conf.projection || /* default projection */ SMap.projUTM33);

      this.getMap().getSignals().makeEvent("layer-change-before", this.getId());
      
      // redraw all marker and geometry layers (so that they are not off when projection changes)
      // - taken from SMap.prototype._redraw
      var map = this.getMap();
     	var cont = map._geometryCanvas.getContainer();
     	cont.style.left = -(map._offset.x+map._sizeHalf.x) + "px";
     	cont.style.top = -(map._offset.y+map._sizeHalf.y) + "px"; 

     	map._layers.forEach(function(layer)
      {
        if (!(layer.getId() in SMap.CONFIG)) /* skip map layers */
          layer.redraw(true);
      });
      
     	if (map._card) { map._card.anchorTo(map._card.getAnchor()); } /* pokud je vizitka, tak jeji ukotveni */
    }
  }
  
  this.enableOrig(this);
});

// ******* smooth out zoom level change (gets nearest zoom of another layer)
SMap.prototype.getNearestZoom = function(l2, options)
{
  options = options || {};

  // get target zoom range
  var zoomRange = SMap.CONFIG[l2].zoom || /* Mapy.cz transparent layers */ [3, 15];
  var minZoom = zoomRange[0];
  var maxZoom = zoomRange[zoomRange.length - 1];
  
  var zoom1 = this.getZoom();
  var p1 = new SMap.Pixel(0, 0);
  var p2 = new SMap.Pixel(100, 0);
  
  var dist1 = p1.toCoords(this, zoom1).distance(p2.toCoords(this, zoom1));

  // set projection of target layer
  var oldProj = this.getProjection();
  this.setProjection2(SMap.CONFIG[l2].projection || SMap.projUTM33);
  
  // find nearest zoom level
  var minDistZoom = zoom1;
  var minDist = 1e10;
    
  for (var i = minZoom; i <= maxZoom; i++)
  {
    var dist2 = p1.toCoords(this, i).distance(p2.toCoords(this, i));

    if (Math.abs(dist2 - dist1) < minDist)
    {
      minDistZoom = i;
      minDist = Math.abs(dist2 - dist1);
    }
  }

  // choose next higher/lower zoom when set in options
  var diff = p1.toCoords(this, minDistZoom).distance(p2.toCoords(this, minDistZoom)) - dist1;
  
  if (options.higher && minDist != 0 && minDistZoom < maxZoom && diff > 0)
    minDistZoom++;
    
  else if (options.lower && minDist != 0 && minDistZoom > minZoom && diff < 0)
    minDistZoom--;

  // restore projection
  this.setProjection2(oldProj);

  return minDistZoom;
}

// get scale (options can have params layer, zoom and round)
SMap.prototype.getScale = function(options)
{
  options = options || {};

  // set projection of target layer
  if (options.layer)
  {
    var oldProj = this.getProjection();
    this.setProjection2(SMap.CONFIG[layer].projection || SMap.projUTM33);
  }
  
  var p1 = new SMap.Pixel(0, 0);
  var p2 = new SMap.Pixel(1, 0);
  var zoom = options.zoom || this.getZoom();
  
  var scale = p1.toCoords(this, zoom).distance(p2.toCoords(this, zoom));

  // restore projection
  if (options.layer)
  {
    this.setProjection2(oldProj);
  }
  
  if (options.round)
  {
    if (scale > 0.9)
      scale = Math.round(scale * 10) * 1000;
    else if (scale > 0.09)
      scale = Math.round(scale * 100) * 100;
    else
      scale = scale * 10000;  
  }
  else
    scale *= 10000;

  return scale;
}

SMap.Control.Copyright.prototype._computeScale = function()
{
  return ("1 : " + this.getMap().getScale({round: true}));

}

// ************
// coord fixes - this object contains LatLon->PP results
// (helps fix incorrect UTM->WGS84 conversion outside Europe/Africe)

// this removes mapy.cz original limitation to Europe and North Africa only
encapsulate(SMap.Coords.prototype, "isValid", function()
{
  var c = this.toWGS84();
  
  // latitude: using +-90 results in texture errors on poles (so use lower values)
  // longitude:  can be as high/low as we desire (we get rotating world)
  return (c[1] > -75 && c[1] < 85 && c[0] > -360 && c[0] < 360); 
});

SMap.Coords.prototype.isValidUTM33 = function()
{
  var c = this.toWGS84();
  return (c[1] > 38 && c[1] < 57 && c[0] > 5 && c[0] < 25); 
}

SMap.Coords.prototype.CoordCenter = {};

// fromWGS84 - add result of conversion to CoordCenter object
encapsulate(SMap.Coords, "fromWGS84", function(latD, lonD)
{
  var ret = SMap.Coords.fromWGS84Orig(latD, lonD);

  // save conversion
  SMap.Coords.prototype.CoordCenter[[ret.x, ret.y]] = [parseFloat(latD), parseFloat(lonD)];
  //log("fromWGS84: " + latD + ", " + lonD + " >>> " + ret.x + "_" + ret.y);
  return ret;
});

// toWGS84 - retrieve correct WGS84 coordinates (if possible)
encapsulate(SMap.Coords.prototype, "toWGS84", function(format)
{
  // convert WGS84->PP previously run, so just retrieve result and format it using line from original function
  var XY = SMap.Coords.prototype.CoordCenter[[this.x, this.y]];
  
  if (typeof XY !== 'undefined')
  {
    //log("toWGS84: " + this + " >>> " + result);
   	var ret = (typeof(format) == "number") ? this._formatWGS84(XY, format) : XY;   
  }
  
  // when in UTM33 maps - no previous WGS84->PP conversion, so run default function
  else
  {
    var ret = this.toWGS84Orig(format);
    //log("Fail toWGS84: " + this + " >>> " + ret);
  }
  
  // export as [lat, lon] and fix longitude outside of +-180
  if (format == "LatLon")
  {
    // retrieve lat/lon
    var lat = ret[1];
    var lon = ret[0];
  
    // longitude may go over 180 in both directions, so account for that
    lon = (lon + 360 * Math.ceil(Math.abs(lon) / 360)) % 360;
    lon = (lon >= 180) ? (lon - 360) : lon;

    ret = [lat, lon];  
  }
  
  return ret;
});

// fromUTM33 - just for monitoring when needed
// encapsulate(SMap.Coords, "fromUTM33", function(x, y)
// {
//  var latlon = SMap.Coords.fromUTM33Orig(x, y);
//  //log("fromUTM33: " + x + "_" + y + " >>> " + latlon);
//  return latlon;
// });

// computeCenterZoom: compute center zoom based on projection
encapsulate(SMap.prototype, "computeCenterZoom", function(arr, usePadding)
{
  var projection = SMap.CONFIG[this.getMap().CurrentMapId].projection;
  
  if (projection && projection !== SMap.projUTM33)
  {
   	/* WGS varianta - copied from map.js*/
   	var minX = Infinity;
   	var minY = Infinity;
   	var maxX = -Infinity;
   	var maxY = -Infinity;
   	for (var i=0;i<arr.length;i++) {
   		var item = arr[i].toWGS84();
   		minX = Math.min(minX, item[0]);
   		minY = Math.min(minY, item[1]);
   		maxX = Math.max(maxX, item[0]);
   		maxY = Math.max(maxY, item[1]);
   	}
   	var x = (minX+maxX)/2;
   	var y = (minY+maxY)/2;
   	
   	/* pocet stupnu, ktere chceme zobrazit */
   	var diffX = maxX-minX;
   	var diffY = maxY-minY;
   	
   	/* kolikatinu sveta potrebujeme zobrazit */
   	var w = this._size.x;
   	var h = this._size.y;
   	if (usePadding) { 
   		w -= this._padding.left;
   		w -= this._padding.right;
   		h -= this._padding.top;
   		h -= this._padding.bottom;
   	}
   
   	var fracX = (360*w)/(SMap.Projection.Mercator.WORLD_SIZE * diffX);
   	var fracY = (180*h)/(SMap.Projection.Mercator.WORLD_SIZE * diffY);
   	var frac = Math.min(fracX, fracY);
   
   	var zoom = Math.floor(Math.log(frac) / Math.LN2);
   
   	return [SMap.Coords.fromWGS84(x,y),zoom];
  }
  else
    return(this.computeCenterZoomOrig(arr, usePadding));
});

// ************
// fix local XHR (for oblique layer)
if (JAK.Browser.client == "ie" && document.protocol == "File Protocol")   
{
  // remove layer 19 for IE8-
  // doesn't work also in IE9 when using MOTW
  // (http://msdn.microsoft.com/en-us/library/ms537628(v=vs.85).aspx)
  if (JAK.Browser.version <= 8)
     SMap.CONFIG.obliqueExtents = [];

  // in IE9+, change XDomainRequest to XMLHttpRequest (otherwise calls will be denied)  
  else
    XDomainRequest = XMLHttpRequest;
}

// note: in webkit, marker title can't coexist with draggable markers (displaying card then often fails)
// (fixed by displaying div on hover instead)

// ************
// addDefaultLayer - add custom build url
encapsulate(SMap.prototype, "addDefaultLayer", function(id)
{
  var layer = this.addDefaultLayerOrig(id);
  layer._buildURLOrig = layer._buildURL;
  layer._buildURL = SMap.CONFIG[id].buildUrl || layer._buildURL;
  layer.redraw = SMap.CONFIG[id].redraw || layer.redraw;  
  return layer;
});

// **** don't render (sub)layers that are outside of zoom bounds
encapsulate(SMap.Layer.Tile.prototype, "_render", function(tiles)
{//log("render - " + this.getId());
  var zoom = this.getMap().getZoom();
  var minmax = SMap.CONFIG[this.getId()].zoom;

  if (minmax && minmax.length && (zoom < minmax[0] || zoom > minmax[minmax.length - 1]))
    return;

  this._renderOrig.apply(this, arguments)
  
  // my code
  if (!this.same)
  {
    var map = this.getMap();
    
    for (var i = 0; i < tiles.length; i++)
    {
      var obj = tiles[i];
      var t = obj.tile;
      var node = obj.node;
      
      if (!node)
        continue;
      
      // tl - top left, br - bottom right, tr - top right; tlp = top left pixel, ...
      // get source corners
      var tl = t.topLeftCoords(map);
      var br = t.bottomRightCoords(map);
      
      var tlp = tl.toPixel(map);
      var brp = br.toPixel(map);
      
      var trp = new SMap.Pixel(brp.x, tlp.y);
      var tr = trp.toCoords(map);
      
      // get target corners
      map.setProjection2(this.projOld);
      var tlp2 = tl.toPixel(map, map._zoom + this.zoomDelta);
      var brp2 = br.toPixel(map, map._zoom + this.zoomDelta);
      var trp2 = tr.toPixel(map, map._zoom + this.zoomDelta);
      map.setProjection2(this.proj);

      // get translation
      var diffX = ((tlp2.x - tlp.x) + (brp2.x - brp.x)) / 2;
      var diffY = ((tlp2.y - tlp.y) + (brp2.y - brp.y)) / 2;
      
      // get scaleX and scaleY
      function dist(a, b) {return Math.sqrt(Math.pow(a.x - b.x, 2) + Math.pow(a.y - b.y, 2))};
      var scaleX = dist(tlp2, trp2) / dist(tlp, trp);
      var scaleY = dist(trp2, brp2) / dist(trp, brp);

      // get rotation
      var angle = 180 + Math.atan2(tlp2.y - trp2.y, tlp2.x - trp2.x) * 180 / Math.PI;
      angle += (map.getOrientation() == SMap.WEST || map.getOrientation() == SMap.EAST) ? 90 : 0;


      node.style[SMap.TRANSFORM] = "translate(" + diffX + "px, " + diffY + "px) scale(" + scaleX + "," + scaleY + ") rotate(" + angle + "deg)";
    }  
  }  
  
});

// **** reproject tiles when layers have incompatible projection

encapsulate(SMap.Layer.Tile.prototype, "redraw", function(full)
{
  if (!this._active)
    return;

//  log(this.getId());

  var map = this.getMap();
  this.proj = SMap.CONFIG[this.getId()].projection || SMap.projUTM33;
  this.projOld = map.getProjection();
  this.same = (this.projOld === this.proj
    /* don't reproject layers from ČÚZK, that can be reprojected directly on WMS server */ 
    || (this._options.noReprojection && this._options.noReprojection(map)));

  // don't reproject anything within the oblique layer
  if (map.getZoom() == 19 && (map.CurrentMapId == SMap.DEF_SMART_OPHOTO 
    || map.CurrentMapId == SMap.DEF_SMART_TURIST || map.CurrentMapId == SMap.DEF_SMART_BASE))
    this.same = true;
  
  // don't reproject when transform CSS property not supported
  if (!SMap.TRANSFORM)
    this.same = true;

  if (!this.same)
  {
    this.zoomDelta = map.getZoom() - map.getNearestZoom(this.getId(),
      // sometimes choose only higher or lower zoom for rendering nicety reasons 
      {higher: this._options.reprojectionZoomHigher, lower: this._options.reprojectionZoomLower});

    map._zoom -= this.zoomDelta;
    map.setProjection2(this.proj);
    
    // increase prefetch to cover entire viewport (in case we scale down or significantly rotate tiles)
    this._options.prefetch += 1;
    
    // remove prefetchDelay
    var prefetchDelayBackup = this._options.prefetchDelay;
    this._options.prefetchDelay = 0;
  }

  var ret = this.redrawOrig(full);
  
  if (!this.same)
  {
    map.setProjection2(this.projOld);   
    map._zoom += this.zoomDelta;
    this._options.prefetch -= 1;
    this._options.prefetchDelay = prefetchDelayBackup;
  }
  
  this.same = false;
  
  return ret;
});


// prevent random error in tile garbage collector
encapsulate(SMap.Layer.Tile.prototype, "_gc", function(tiles)
{
  try{this._gcOrig.apply(this, arguments)} catch(ex){};
});

/*** fix listener errors, when we double click on a marker ***/
encapsulate(SMap.prototype, "animate", function(c, options)
{
  if (JAK.Timekeeper.getInstance()._findListener(this) == -1)
    return (this.animateOrig(c, options));
});

encapsulate(SMap.prototype, "setZoom", function(z, fixedPoint, animate)
{
  if (JAK.Timekeeper.getInstance()._findListener(this) == -1)  
    return (this.setZoomOrig(z, fixedPoint, animate));
});


// ************
// SMap.Control.Selection - modify to right click


// set more params
encapsulate(SMap.Control.Selection.prototype, "$constructor", function(thickness)
{
  // call and backup $super()
  this.$super();
  var temp = this.$super;
  this.$super = function(){};
  
  // note: Firefox and SHIFT+right click launches app context menu (can't be overridden)
  // - customStartDrag - called on mousedown
  // - customRedraw - called on mousemove 
  // - customHandler - called on draw finish (mouse up)
  // - customFailDrag - called on finish, but when mouse movement was too small and customHandler wouldn't be triggered
  var options = (typeof thickness === 'object') ? thickness : {thickness: thickness};
  this.customHandler = options.customHandler || null;
  this.fillColor = options.fillColor || null;
  this.cursor = options.cursor || null;
  this.customId = options.customId || null;
  this.shift = options.shift || false;
  this.ctrl = options.ctrl || false;
  this.alt = options.alt || false;
  this.meta = options.meta || false;
  this.mouseButton = (typeof options.mouseButton !== 'undefined') ? options.mouseButton : JAK.Browser.mouse.right;
  this._backupRedraw = this._redraw;   // enable us to call original redraw from our functions
  this._redraw = options.customRedraw || this._redraw;
  this.customStartDrag = options.customStartDrag || null;
  this.customFailDrag = options.customFailDrag || null; // called when dragging fails (no mouse movement)
  
  // call main function
  this.$constructorOrig(options.thickness);
  
  // restore $super()
  this.$super = temp;
});

// start selection only on dragging content (not any controls)
encapsulate(SMap.Control.Selection.prototype, "setOwner", function(owner)
{
  this.$super(owner);
  var elm = owner.getContent();
  this._ec.push(JAK.Events.addListener(elm, "mousedown", this, "_mousedown"));
});

// react on right click
encapsulate(SMap.Control.Selection.prototype, "_mousedown", function(e, elm)
{ 
  // handle special keys (allowing for custom handling)
  if (this.shift != !!e.shiftKey || this.ctrl != !!e.ctrlKey || this.alt != !!e.altKey || this.meta != !!e.metaKey
  || this.mouseButton != e.button)
    return;

  // Maxthon 2 fires mousedown on mouseup (on right click with SHIFT etc. pressed), prevent it
  if (this.mouseIsDown)
    return false;

  this.mouseIsDown = true;

  // Opera
  if (JAK.Browser.client == "opera")
    EventUtil.preventDefault(e);

  // custom function on start
  if (this.customStartDrag)
    this.customStartDrag(e, elm);

  // change cursor
  this.getMap().setCursor(this.cursor || "pointer");
 
  // *** prevent context menu from appearing ***
  var that = this;
  
  this.handleMouseUp = function(e, target)
  {
    if (that._point2 !== that._point1)
    {
      EventUtil.stopPropagation(e);
      EventUtil.preventDefault(e);  
    }

    if (e.type == "contextmenu")
    {
      while (that.contextMenuHandlers.length)
        EventUtil.removeHandler(that.contextMenuHandlers.pop());
    }
    
    else if (e.type == "mouseup")
    {
      while (that.mouseUpMenuHandlers.length)
        EventUtil.removeHandler(that.mouseUpMenuHandlers.pop());

      // remove handler for context menu here as well
      if (that.mouseButton != JAK.Browser.mouse.right)
      {
        while (that.contextMenuHandlers.length)
          EventUtil.removeHandler(that.contextMenuHandlers.pop());
      }
  
      // call mouseup function (only if any move happened)
      if (that.movedMouse)
        that._mouseup.call(that, e, target);
    }
  }
 
  // prevent context menu bubbling
  // IE7-8 --> guess map layer index (should be 0), IE9+ - use capture mode
  this.contextMenuHandlers = [];
  
  for (var i = 0; i < elm.childNodes.length; i++)
    this.contextMenuHandlers.push(EventUtil.addHandler(elm.childNodes[i], "contextmenu", this.handleMouseUp, false));

  this.contextMenuHandlers.push(EventUtil.addHandler(document.documentElement, "contextmenu", this.handleMouseUp, true));
  
  // prevent middle click bubbling (note: don't do this in IE7-8, would cause a bug)
  this.mouseUpMenuHandlers = [];
  this.mouseUpMenuHandlers.push(EventUtil.addHandler(document.documentElement, "mouseup", this.handleMouseUp, true));

  // *** call original function (e can't be modified, so create a new one) ***
  this._mousedownOrig({button: JAK.Browser.mouse.left, ctrlKey: 1, metaKey: 0, clientX: e.clientX, clientY: e.clientY}, elm);
  
  // change fill color (if set)
  if (this.fillColor)
    this._dom.container.style.backgroundColor = this.fillColor;
});

// mouse up
encapsulate(SMap.Control.Selection.prototype, "_mouseup", function(e, elm)
{
  EventUtil.preventDefault(e);
  EventUtil.stopPropagation(e);
  this.movedMouse = false;
  this.mouseIsDown = false;  /* Maxthon 2 fix */
  
  // restore cursor
  this.getMap().setCursor(null);

  // custom handler
  if (this.customHandler)
  {
    // get left-bottom/right-top points and direction where we started
    var direction = ((this._point1.x <= this._point2.x) ? "l" : "r") + ((this._point1.y <= this._point2.y) ? "t" : "b")
    var p1 = new SMap.Pixel(Math.min(this._point1.x, this._point2.x), Math.min(this._point1.y, this._point2.y));
    var p2 = new SMap.Pixel(Math.max(this._point1.x, this._point2.x), Math.max(this._point1.y, this._point2.y));

    // original cleanup
  	this._ecTmp.forEach(JAK.Events.removeListener, JAK.Events);
  	this._ecTmp = [];
   	this._hide();    

    // make sure something is selected
    var selectedSomething = (p1.x != p2.x && p1.y != p2.y);
    
    if (selectedSomething)
      this.customHandler(p1, p2, this.customId, direction);
    else if (this.customFailDrag)
      this.customFailDrag(p1, p2, this.customId, direction);
  }
  
  // default: zoom to selection
  else
  {
    this._mouseupOrig(e, elm);
  }
});

// handle mouse move
encapsulate(SMap.Control.Selection.prototype, "_mousemove", function(e, elm)
{
  this.movedMouse = true;
  return(this._mousemoveOrig(e, elm));
})

// stop moving map using mouse if alt or shift is pressed (to enable setting selection on any mouse/key combination)
encapsulate(SMap.Control.Mouse.prototype, "_mouseDown", function(e, elm)
{
  if (e.shiftKey || e.altKey)
    return;
    
  return(this._mouseDownOrig(e, elm));
});

//**********************
// implement project point function
// from: http://www.movable-type.co.uk/scripts/latlong.html
SMap.Coords.prototype.projectPoint = function(azimuth, distance)
{
  var latlon = this.toWGS84();

  var lat1 = latlon[1] * Math.PI / 180;
  var lon1 = latlon[0] * Math.PI / 180;
  
  var R = 6371000;
  var d = distance/R;  // d = angular distance covered on earth’s surface

  var brng = azimuth * Math.PI / 180;

  var dLat = d*Math.cos(brng);
  // nasty kludge to overcome ill-conditioned results around parallels of latitude:
  if (Math.abs(dLat) < 1e-10) dLat = 0; // dLat < 1 mm
  
  var lat2 = lat1 + dLat;
  var dPhi = Math.log(Math.tan(lat2/2+Math.PI/4)/Math.tan(lat1/2+Math.PI/4));
  var q = (isFinite(dLat/dPhi)) ? dLat/dPhi : Math.cos(lat1);  // E-W line gives dPhi=0
  var dLon = d*Math.sin(brng)/q;
  
  // check for some daft bugger going past the pole, normalise latitude if so
  if (Math.abs(lat2) > Math.PI/2) lat2 = lat2>0 ? Math.PI-lat2 : -Math.PI-lat2;
  
  lon2 = (lon1+dLon+3*Math.PI)%(2*Math.PI) - Math.PI;
 
  lat2 *= 180 / Math.PI;
  lon2 *= 180 / Math.PI;

  //log(this.distance(SMap.Coords.fromWGS84(lon2, lat2)) - distance);
  return SMap.Coords.fromWGS84(lon2, lat2);
}

// function to find out if current map is UTM33 based
SMap.prototype.isUTM33 = function()
{
  var projection = SMap.CONFIG[this.CurrentMapId].projection;
  return (!projection || projection === SMap.projUTM33) ? true : false;
}

// switches map (and updates layer switch)
SMap.prototype.switchMap = function(id)
{
  JAK.DOM.removeClass(document.querySelector(".layer-switch .active"), "active");
  this.getLayer(this.CurrentMapId).disable();
  this.CurrentMapId = id;
  this.getLayer(this.CurrentMapId).enable();
}

// fix distance calculation for far away places
encapsulate(SMap.Coords.prototype, "distance", function(target, altitude)
{
  if (!this.isValidOrig() || !target.isValidOrig())
  {
    var latlon = this.toWGS84("LatLon");
    var latlon2 = target.toWGS84("LatLon");
   
    var lat1 = latlon[0];
    var lon1 = latlon[1];
   
    var lat2 = latlon2[0];
    var lon2 = latlon2[1]; 
   
    var R = 6371; // km
    var dLat = (lat2-lat1) * Math.PI / 180;
    var dLon = (lon2-lon1) * Math.PI / 180;
    var lat1 = lat1 * Math.PI / 180;
    var lat2 = lat2 * Math.PI / 180;
    
    var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
            Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1) * Math.cos(lat2); 
    var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)); 
    var d = R * c;
  
    return d * 1000;  
  }

  // use original calculation for local places
  else
    return(this.distanceOrig(target, altitude));
});

// send redraw event also before the fact, not only after it
encapsulate(SMap.prototype, "_redraw", function(full)
{
  this._signals.makeEvent("map-redraw-before", this, {full:full});
  return(this._redrawOrig(full));
});

// this functions sometimes fails in IE7 and maybe IE8 - attempt to fix it
encapsulate(JAK.VML.prototype, "setCenterRadius", function(element, center, radius)
{
  try
  {
    return(this.setCenterRadiusOrig(element, center, radius));
  }
  catch(ex){/*log("JAK.VML setCenterRadius error");*/};
});

// global disabling/enabling of marker dragging (fixes some bugs in free draw mode)
SMap.prototype.disableMarkerDragging = function()
{
  this.markerDraggingDisabled = true;
  
  this.markerDraggingFnBackup = SMap.Marker.Feature.Draggable.prototype._dragStart;
  SMap.Marker.Feature.Draggable.prototype._dragStart = function(){};
}

SMap.prototype.enableMarkerDragging = function()
{
  if (!this.markerDraggingFnBackup)
    return;

  this.markerDraggingDisabled = false;
  SMap.Marker.Feature.Draggable.prototype._dragStart = this.markerDraggingFnBackup;
}

/****** add marker dblclick ******/
encapsulate(SMap.Layer.Marker.prototype, "_build", function()
{
  this._buildOrig();
  this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_ACTIVE], "dblclick", this, "_dblclick"));
  this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_MARKER], "dblclick", this, "_dblclick"));
});

SMap.Layer.Marker.prototype._dblclick = function(e, elm) 
{
 	var t = JAK.Events.getTarget(e);
 	var tops = [];
 	for (var p in this._dom.container) { tops.push(this._dom.container[p]); }
 	
 	for (var id in this._markers) {
 		var m = this._markers[id].marker;
 		var node = t;
 		var a = m.getActive();
 		do {
 			if (node == a) 
       {
 				m.dblclick(e, elm, "x");
 				return;
 			}
 			node = node.parentNode;
 		} while (tops.indexOf(node) == -1);
 	}
}

SMap.Marker.prototype.dblclick = function(e, elm)
{
  this.getMap().getSignals().makeEvent("marker-dblclick", this, {event:e});
};


//****************************************************************************
// implement ability to be able to retrieve coordinates from Tile

/* get topLeft and bottomRight coords for both UTM33 and Mercator */
SMap.Tile.prototype.topLeftCoords = function(map, projection, zoom)
{
  var ts = this.tileSize;
  var proj = projection || map.getProjection(); 
  var coef = proj._screenCoefficients;
  var x = this.x * ts;
  var y = this.y * ts + ts * (coef.y == 1 ? 1 : 0);
  
  var coord = proj._absolutePixelToCoords(new SMap.Pixel(x, y), zoom || map.getZoom());
  return coord; 
}

SMap.Tile.prototype.bottomRightCoords = function(map, projection, zoom)
{
  var proj = projection || map.getProjection();
  var coef = proj._screenCoefficients;
  var px = new SMap.Pixel(this.tileSize * coef.x /* test this when we have map with -x */, this.tileSize * coef.y * -1);
  var coord = proj.shiftCoords(this.topLeftCoords(map), zoom || map.getZoom(), px);
  return coord;
}

SMap.Tile.prototype.getBBOX = function(map, version)
{
  var proj = map.getProjection();
  var convertTo;
  
  // UTM33
  if (proj.EPSG == 32633)
    convertTo = "toUTM33";
    
  // WGS84
  else if (proj.EPSG == 4326)
    convertTo = "toWGS84";
    
  // S-JTSK
  else if (proj.EPSG == 102067)
    convertTo = "toSJTSK";
  else
    return "";
    
  var param = "";
  
  // 1.1.1 requires Lon,Lat; new versions require Lat,Lon
  if (proj.EPSG == 4326 && version != "1.1.1")
    param = "LatLon"; 
    
  var tl = (this.topLeftCoords(map))[convertTo](param);
  var br = (this.bottomRightCoords(map))[convertTo](param);
  
  return (tl + "," + br); 
}

//******************
// support contexmenu event for geometry-click

encapsulate(SMap.Layer.Geometry.prototype, "_build", function()
{
  // original
 	this._canvas = JAK.Vector.getCanvas(1, 1); /* tenhle canvas pouzijeme jen pro vyrobu prvku, ale cpeme je do rodicovskeho */
 	var group = this._canvas.group();
 	group.id = this.id+"-"+SMap.LAYER_GEOMETRY;
 
 	this._ec.push(JAK.Events.addListener(group, "click", this, "_click"));
 	this._ec.push(JAK.Events.addListener(group, "mousedown", this, "_mouseDown"));
 
 	this._dom.container[SMap.LAYER_GEOMETRY] = group;
 	this._canvas.setContent(group);

  // our
  this._ec.push(JAK.Events.addListener(group, "contextmenu", this, "_click"));
});

//**************
// attempt to improve loading speed of ZM10 tiles
encapsulate(SMap.Layer.Tile.prototype, "_buildElement", function(tile)
{
  var node = this._buildElementOrig(tile);
 
  if (node)
  {
    node.onerror = function()
    {
      if (/cuzk/.test(node.src))
      {
        node.src = node.src + "&rand=" + Math.random();
        node.onerror = null; //function() {node.style.display = "none"};
      }
      
      // hide altogether on error
      else
      {
        node.style.display = "none";
      }
    };  
  }
  
  //log_object(this._options);
  
  if ("opacity" in this._options)
  {
    node.style.opacity = this._options.opacity;

    // doesn't work (will probably work in the future, when seznam.cz add ability to change opacity)
    if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
      node.style.filter = "alpha(opacity=" + (this._options.opacity * 100) + ")";
  }
  
  return node;
});

//******************************************************
// format coordinates to a string

// type: WGS84, SJTSK
// param: WGS84 - deg, min, sec, SJTSK - N/A
SMap.Coords.prototype.format = function(type, param)
{
  if (type == "WGS84")
  {
    function getDegrees(c) {return (Math.floor(Math.abs(c)));}
    function getMinutes(c) {return ((Math.abs(c) - getDegrees(c)) * 60);}
    function getSeconds(c) {return ((getMinutes(c) - Math.floor(getMinutes(c))) * 60);}  

    var point = this.toWGS84("LatLon");
    var lat = point[0];
    var lon = point[1];

    // get seconds
    if (/sec/.test(param))
    {
      lat = ((lat > 0) ? "N" : "S") + " " + getDegrees(lat) + "° " + Math.floor(getMinutes(lat)) + "' " + getSeconds(lat).toFixed(2) + "\"";
      lon = ((lon > 0) ? "E" : "W") + " " + getDegrees(lon) + "° " + Math.floor(getMinutes(lon)) + "' " + getSeconds(lon).toFixed(2) + "\"";
    }

    // get minutes
    else if (/min/.test(param))
    {
      lat = ((lat > 0) ? "N" : "S") + " " + getDegrees(lat) + "° " + getMinutes(lat).toFixed(3);
      lon = ((lon > 0) ? "E" : "W") + " " + getDegrees(lon) + "° " + getMinutes(lon).toFixed(3);
    }
    
    // get degrees
    else //if (/deg/.test(param))
    {
      lat = lat.toFixed(7);
      lon = lon.toFixed(7);
    }        
  }
  
  else if (type == "SJTSK")
  {
    var sjtsk = this.toSJTSK();
    var lat = "Y " + sjtsk[0].toFixed(3);
    var lon = "X " + sjtsk[1].toFixed(3);  
  }
  
  else
    return "";
    
  return (lat + "  " + lon);
}

//******************************************************
// functions for S-JTSK <-> WGS84 conversion (EPSG 102067)
SMap.Coords.prototype.toSJTSK = function()
{
  var wgs = this.toWGS84();
  var lat = wgs[1];
  var lon = wgs[0];
  
  function sqr(x) {return x * x;}

  var d2r = Math.PI / 180;
  var a = 6378137.0;
  var f1 = 298.257223563;
  var dx = -570.69;
  var dy = -85.69;
  var dz = -462.84;
  var wx = 4.99821/3600 * Math.PI/180;
  var wy = 1.58676/3600 * Math.PI/180;
  var wz = 5.2611/3600 * Math.PI/180;
  var m  = -3.543e-6;

  var B = lat * d2r;
  var L = lon * d2r;
  var H = 200;
  
  var e2 = 1 - sqr(1 - 1 / f1);
  var rho = a / Math.sqrt(1 - e2 * sqr(Math.sin(B)));
  var x1 = (rho + H) * Math.cos(B) * Math.cos(L);
  var y1 = (rho + H) * Math.cos(B) * Math.sin(L);
  var z1 = ((1 - e2) * rho + H) * Math.sin(B);
  
  var x2 = dx + (1 + m) * (x1 + wz * y1 - wy * z1);
  var y2 = dy + (1 + m) * ( - wz * x1 + y1 + wx * z1);
  var z2 = dz + (1 + m) * (wy * x1 - wx * y1 + z1);
  
  a = 6377397.15508;
  f1 = 299.152812853;
  var ab = f1 / (f1 - 1);
  var p = Math.sqrt(sqr(x2) + sqr(y2));
  e2 = 1 - sqr(1 - 1 / f1);
  var th = Math.atan(z2 * ab / p);
  var st = Math.sin(th);
  var ct = Math.cos(th);
  var t = (z2 + e2 * ab * a * (st * st * st)) / (p - e2 * a * (ct * ct * ct));
  
  B = Math.atan(t);
  H = Math.sqrt(1 + t * t) * (p - a / Math.sqrt(1 + (1 - e2) * t * t));
  L = 2 * Math.atan(y2 / (p + x2));
  
  a = 6377397.15508;
  var e = 0.081696831215303;
  var n = 0.97992470462083;
  var rho0 = 12310230.12797036;
  var sinUQ = 0.863499969506341;
  var cosUQ = 0.504348889819882;
  var sinVQ = 0.420215144586493;
  var cosVQ = 0.907424504992097;
  var alpha = 1.000597498371542;
  var k2 = 1.00685001861538;
  
  var sinB = Math.sin(B);
  t = (1 - e * sinB) / (1 + e * sinB);
  t = sqr(1 + sinB) / (1 - sqr(sinB)) * Math.exp(e * Math.log(t));
  t = k2 * Math.exp(alpha * Math.log(t));
  
  var sinU = (t - 1) / (t + 1);
  var cosU = Math.sqrt(1 - sinU * sinU);
  var V = alpha * L;
  var sinV = Math.sin(V);
  var cosV = Math.cos(V);
  var cosDV = cosVQ * cosV + sinVQ * sinV;
  var sinDV = sinVQ * cosV - cosVQ * sinV;
  var sinS = sinUQ * sinU + cosUQ * cosU * cosDV;
  var cosS = Math.sqrt(1 - sinS * sinS);
  var sinD = sinDV * cosU / cosS;
  var cosD = Math.sqrt(1 - sinD * sinD);
  
  var eps = n * Math.atan(sinD / cosD);
  rho = rho0 * Math.exp( - n * Math.log((1 + sinS) / cosS));
  
  var CX = rho * Math.sin(eps);
  var CY = rho * Math.cos(eps);

  return [-CX, -CY];    
}

// SMap.Coords.fromEPSG(a, b, EPSG)
// {
//   if (EPSG == 4326)
//     return (this.fromWGS84(a, b, EPSG));
//   if (EPSG == 201067)
//     return (this.fromSJTSK(a, b, EPSG));
// 
// }

SMap.Coords.fromSJTSK = function(x, y)
{
  /* vstupní údaje */
  var X = x;
  var Y = y;
  var H = 45;
  
  /* Vypocet zemepisnych souradnic z rovinnych souradnic */ 
  var a = 6377397.15508;
  var e = 0.081696831215303;
  var n = 0.97992470462083;
  var konst_u_ro = 12310230.12797036;
  var sinUQ = 0.863499969506341;
  var cosUQ = 0.504348889819882;
  var sinVQ = 0.420215144586493;
  var cosVQ = 0.907424504992097;
  var alfa = 1.000597498371542;
  var k = 1.003419163966575;
  var ro = Math.sqrt(X * X + Y * Y);
  var epsilon = 2 * Math.atan(Y / (ro + X));
  var D = epsilon / n;
  var S = 2 * Math.atan(Math.exp(1 / n * Math.log(konst_u_ro / ro))) - Math.PI / 2;
  var sinS = Math.sin(S);
  var cosS = Math.cos(S);
  var sinU = sinUQ * sinS - cosUQ * cosS * Math.cos(D);
  var cosU = Math.sqrt(1 - sinU * sinU);
  var sinDV = Math.sin(D) * cosS / cosU;
  var cosDV = Math.sqrt(1 - sinDV * sinDV);
  var sinV = sinVQ * cosDV - cosVQ * sinDV;
  var cosV = cosVQ * cosDV + sinVQ * sinDV;
  var Ljtsk = 2 * Math.atan(sinV / (1 + cosV)) / alfa;
  var t = Math.exp(2 / alfa * Math.log((1 + sinU) / cosU / k));
  var pom = (t - 1) / (t + 1);

  do
  {
    var sinB = pom;
    pom = t * Math.exp(e * Math.log((1 + e * sinB) / (1 - e * sinB))); 
    pom = (pom - 1) / (pom + 1);
  } while (Math.abs(pom - sinB) > 1e-15);
  
  var Bjtsk = Math.atan(pom / Math.sqrt(1 - pom * pom));


  /* Pravoúhlé souřadnice ve S-JTSK */  
  a = 6377397.15508;
  f_1 = 299.152812853;
  e2 = 1 - (1 - 1 / f_1) * (1 - 1 / f_1);
  ro = a / Math.sqrt(1 - e2 * Math.sin(Bjtsk) * Math.sin(Bjtsk));
  x = (ro + H) * Math.cos(Bjtsk) * Math.cos(Ljtsk); 
  y = (ro + H) * Math.cos(Bjtsk) * Math.sin(Ljtsk); 
  z = ((1 - e2) * ro + H) * Math.sin(Bjtsk);
  
  /* Pravoúhlé souřadnice v WGS-84 */ 
  var dx = 570.69;
  var dy = 85.69;
  var dz = 462.84; 
  var wz = -5.2611 / 3600 * Math.PI / 180;
  var wy = -1.58676 / 3600 * Math.PI / 180;
  var wx = -4.99821 / 3600 * Math.PI / 180;
  var m = 3.543e-6; 
  xn = dx + (1 + m) * (x + wz * y - wy * z);
  yn = dy + (1 + m) * (-wz * x + y + wx * z);
  zn = dz + (1 + m) * (wy * x - wx * y + z);
  
  /* Geodetické souřadnice v systému WGS-84 */ 
  a = 6378137.0;
  f_1 = 298.257223563;
  a_b = f_1 / (f_1 - 1);
  var p = Math.sqrt(xn * xn + yn * yn);
  var e2 = 1 - (1 - 1 / f_1) * (1 - 1 / f_1);
  var theta = Math.atan(zn * a_b / p);
  var st = Math.sin(theta);
  var ct = Math.cos(theta);
  var t = (zn + e2 * a_b * a * st * st * st) / (p - e2 * a * ct * ct * ct);
  B = Math.atan(t);
  L = 2 * Math.atan(yn / (p + xn));
  H = Math.sqrt(1 + t * t) * (p - a / Math.sqrt(1 + (1 - e2) * t * t));
  
  return (new this.fromWGS84(L / Math.PI * 180, B / Math.PI * 180));
}

SMap.Coords.fromPuwg1992 = function(y, x)
{
  var N = (8.3206362430206483E-1 + 1.3481879090606402E-8 * y + 1.5602327664852516E-7 * x - 1.3256276739522121E-14 * y * y + 4.1825950896992127E-15 * x * y - 2.9377381449595037E-16 * x * x - 4.5218702489875804E-22 * y * y * y - 3.9752390006101247E-21 * x * y * y + 6.7313812941904462E-22 * x * x * y - 5.6886908481842923E-23 * x * x * x + 2.2731624246717339E-28 * y * y * y * y - 4.113938338560179E-28 * x * y * y * y - 6.783150621322181E-28 * x * x * y * y + 2.4412235774740029E-28 * x * x * x * y + 5.4276419199397539E-31 * x * x * x * x - 6.4388670061403155E-37 * y * y * y * y * y + 2.0368288420261601E-34 * x * y * y * y * y + 4.393095690025773E-36 * x * x * y * y * y - 2.4158699483191763E-34 * x * x * x * y * y - 1.3676810703780487E-36 * x * x * x * x * y - 6.1662070346142104E-37 * x * x * x * x * x) * 180 / Math.PI;
  var E = (2.1529904669561182E-1 + 2.309998186000915E-7 * y - 1.9968132289432323E-8 * x + 4.8036254777118241E-15 * y * y + 3.906197662448301E-14 * x * y - 4.8275072391845386E-15 * x * x - 2.9467755247811066E-21 * y * y * y + 2.6372320268815168E-21 * x * y * y + 8.9030553688988987E-21 * x * x * y - 8.6938915861748507E-22 * x * x * x - 3.7919111897204254E-28 * y * y * y * y - 1.7774342020473573E-27 * x * y * y * y + 2.2263178567053806E-27 * x * x * y * y + 1.7588476236488152E-27 * x * x * x * y - 3.6555719566702145E-28 * x * x * x * x + 1.4971364431168086E-34 * y * y * y * y * y + 1.0721251670702239E-35 * x * y * y * y * y - 1.464901003129573E-33 * x * x * y * y * y - 2.1018265780491216E-35 * x * x * x * y * y + 7.2158604249699042E-34 * x * x * x * x * y + 2.0298976445575483E-36 * x * x * x * x * x) * 180 / Math.PI;
  return (new this.fromWGS84(E, N));
}
  
SMap.Coords.prototype.toPuwg1992 = function()
{
  var wgs = this.toWGS84();
  var x = wgs[1] * Math.PI / 180;
  var y = wgs[0] * Math.PI / 180;
  var cx = -1.6873184841158912E+6 + 7.1740308080610931E+6 * y + 8.4089827999949455E+4 * x - 2.4514780236475319E+6 * y * y - 1.4882979362675548E+6 * x * y + 1.1667953853476048E+6 * x * x + 2.3836063858750649E+6 * y * y * y + 4.7725105592213273E+6 * x * y * y - 2.6700937784006E+6 * x * x * y - 2.0077724548304081E+5 * x * x * x + 1.6544340109570697E+5 * y * y * y * y - 4.7726196774263009E+6 * x * y * y * y - 2.0564038970388323E+6 * x * x * y * y + 1.7055551624134183E+5 * x * x * x * y + 1.9087881651997566E+4 * x * x * x * x - 1.0146215099524893E+5 * y * y * y * y * y + 3.0430736431479454E+3 * x * y * y * y * y + 2.037681378207285E+6 * x * x * y * y * y + 1.0726645753487945E+4 * x * x * x * y * y + 1.8054976024166495E+5 * x * x * x * x * y - 1.7430914509907365E+4 * x * x * x * x * x;
  var cy = -5.2925396175019704E+6 + 3.246150581805408E+5 * y + 6.5982573128608167E+6 * x - 8.4372481974698603E+4 * y * y - 3.9454544085918069E+6 * x * y + 3.7411205534934998E+5 * x * x - 1.2257334379010648E+6 * y * y * y + 5.5874140706162751E+6 * x * y * y + 2.9854418101193309E+6 * x * x * y - 8.7192706190568209E+5 * x * x * x + 9.2642040718702413E+5 * y * y * y * y + 1.1050736306678504E+6 * x * y * y * y - 4.5096171307944506E+6 * x * x * y * y - 3.3980037700653076E+5 * x * x * x * y + 5.1689369445577264E+5 * x * x * x * x - 4.0153157295100391E+2 * y * y * y * y * y - 8.3754242486413941E+5 * x * y * y * y * y + 3.2360315524674952E+3 * x * x * y * y * y + 5.172036182330288E+5 * x * x * x * y * y - 8.9049409680813551E+2 * x * x * x * x * y - 1.1561819681824744E+5 * x * x * x * x * x;
  return [cx, cy];
}

SMap.Projection.ArcGIS = JAK.ClassMaker.makeClass(
{
	NAME: "SMap.Projection.ArcGIS",
 	VERSION: "1.0",
 	EXTEND: SMap.Projection
});

SMap.Projection.ArcGIS.prototype.$constructor = function(tileInfo, fullExtent, delta)
{
  this._owner = null;
  
  this.tileInfo = tileInfo;
  this.fullExtent = fullExtent;

  this.EPSG = this.tileInfo.spatialReference.wkid;
  
  // offset to normal zoom (so that we don't start with 0)
  this.delta = delta || 8;
  
  this._screenCoefficients = {x:1, y:1};  
}

SMap.Projection.ArcGIS.prototype._absolutePixelToCoords = function(pixel, zoom)
{
  var pixelSize = this.tileInfo.lods[zoom - this.delta].resolution;
  var x = pixel.x * pixelSize + this.tileInfo.origin.x;
  var y = -pixel.y * pixelSize + this.tileInfo.origin.y;

  // S-JTSK
  if (this.EPSG == 102067)
    return SMap.Coords.fromSJTSK(-y, -x);
    
  // Puwg1992 (Poland)
  else if (this.EPSG == 2180)
    return SMap.Coords.fromPuwg1992(x, y);
}

SMap.Projection.ArcGIS.prototype._coordsToAbsolutePixel = function(coords, zoom)
{
  if (zoom - this.delta < 0)
    throw("zoom error");

  if (this.EPSG == 102067)  
    var coord = coords.toSJTSK();
  else if (this.EPSG == 2180)
    var coord = coords.toPuwg1992();

 //   try{
  var pixelSize = this.tileInfo.lods[zoom - this.delta].resolution;
 //  } catch(ex){alert(zoom + " / " + this.delta + " / " + this.getMap().getProjection().EPSG)};

  var x = (coord[0] - this.tileInfo.origin.x) / pixelSize;
  var y = -(coord[1] - this.tileInfo.origin.y) / pixelSize;

  return new SMap.Pixel(Math.round(x), Math.round(y)); 
}

//-------------
// set EPSG for standard projections
SMap.Projection.Mercator.prototype.EPSG = 4326;
SMap.Projection.UTM33.prototype.EPSG = 32633;

// create default projections (so that we don't have to create them each time we change a map
SMap.projUTM33 = new SMap.Projection.UTM33();
SMap.projMercator = new SMap.Projection.Mercator();


//------------------------
//******************************************
SMap.Bubble = JAK.ClassMaker.makeClass
({
  NAME: "SMap.Bubble",
  VERSION: "1.0",
  IMPLEMENT: [SMap.IOwned],
  DEPEND: [
  {
  	sClass: JAK.Window,
  	ver: "2.0"
  }]
});
 
SMap.Bubble.prototype.$constructor = function(options)
{
  // set options
  this.options = 
  {
    html: "",
    background: "#C4E2F7",
    fontSize: 12,
    minWidth: 100,
    timeout: 100 /* bubble is hidden after mouseout after timeout ms */  
  }
  
  for (var param in (options || {}))
    this.options[param] = options[param];
  
  // create bubble div
  this.dom = JAK.mel("div", {className: "markerBubble"},
    {display: "none", background: this.options.background, fontSize: this.options.fontSize + "px",
    minWidth: this.options.minWidth + "px"});
  
  this.dom.innerHTML = this.options.html;
}

/* set bubble HTML */
SMap.Bubble.prototype.setHTML = function(html)
{
  this.options.html = html || "";
  this.dom.innerHTML = this.options.html;
}

/* force show */
SMap.Bubble.prototype.show = function()
{
  this.dom.style.display = "";
  this.afterMouseOut = false;
  
  // old IE would select all text on the page --> prevent that
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 9)
    document.selection.empty();  
}

/* force hide */
SMap.Bubble.prototype.hide = function()
{
  this.dom.style.display = "none";
}

SMap.Marker.Feature.Bubble = JAK.ClassMaker.makeSingleton(
{
  NAME: "SMap.Marker.Feature.Bubble",
  VERSION: "1.0",
  EXTEND: JAK.AbstractDecorator
});
 
SMap.Marker.Feature.Bubble.prototype.decorate = function(marker, bubble)
{
  this.$super(marker);

  // bubble already exists?
  if (marker.bubble)
    throw("Marker already has a bubble!");
                                  
  marker.bubble = bubble;

  // add bubble div to marker
  var markerDiv = marker.getContainer()[SMap.LAYER_MARKER];
  markerDiv.appendChild(bubble.dom);

  // set display/hide handlers
  marker._bubbleOver = this._mouseOver;
  marker._bubbleOut = this._mouseOut;
  marker._ec.bubble_over = JAK.Events.addListener(markerDiv, "mouseover", marker, "_bubbleOver");
  marker._ec.bubble_out = JAK.Events.addListener(markerDiv, "mouseout", marker, "_bubbleOut");
  
  marker.oldZIndex = NaN;
  return marker;
}

/* mouseover - show */
SMap.Marker.Feature.Bubble.prototype._mouseOver = function(e, elm)
{
  this.bubble.dom.style.display = "";
  
  // old IE would select all text on the page --> prevent that
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 9)
    document.selection.empty();
  
  /* timeout after mouseout */
  this.bubble.afterMouseOut = false;
  
  // beef up marker z-index, so that the bubble is always on top
  var markerDiv = this.getContainer()[SMap.LAYER_MARKER];
  
  if (isNaN(this.oldZIndex))
    this.oldZIndex = markerDiv.style.zIndex;

  markerDiv.style.zIndex = 10;
}

/* mouseout - hide */
SMap.Marker.Feature.Bubble.prototype._mouseOut = function(e, elm)
{
  this.bubble.afterMouseOut = true;
  var that = this;

  window.setTimeout(function()
  {
    if (that.bubble.afterMouseOut)
    {
      that.bubble.dom.style.display = "none";
      
      // restore marker z-index
      if (!isNaN(that.oldZIndex))
      {
        that.getContainer()[SMap.LAYER_MARKER].style.zIndex = that.oldZIndex;
        that.oldZIndex = NaN;
      }      
    }
  }, this.bubble.options.timeout);
}