// documentation for Street View:
// https://developers.google.com/maps/documentation/javascript/streetview#StreetViewService
// https://developers.google.com/maps/documentation/javascript/reference
// https://developers.google.com/maps/documentation/javascript/v2/reference

translations["_flash_blocked_warning"] = 
{
  "Czech": "" +
    "Níže uvedený text si raději zkopírujte do schránky, protože se znovu zobrazí až v příští verzi mapy.\n" +
    "\n" +
    "Spustili jste StreetView verzi 2, která je svižnější a umožňuje přepnutí do full-screenu.\n" +
    "\n" +
    "Tato verze se ale obvykle potýká s problémy. Je to dáno tím, že používá Flash a ten má jistá nepříjemná omezení. Nefunguje synchronizace s mapou a při změně velikosti okénka StreetView se poloha resetuje do poslední známé pozice. Vyřešit tyto problémy lze tak, že pravým tlačítkem kliknete na flash (tedy na okénko Street View nebo na jakýkoli flash kdekoli jinde v rámci prohlížeče). Ujistěte se, že máte moderní verzi Flashe (alespoň 11.5), kdyžtak updatujte. Pak klikněte na \"Global Settings\" (Globální nastavení), v něm pak dále na \"Advanced\" (Pokročilé) a nakonec na \"Trusted Location Settings\" (přeposlední tlačítko dole). Tam musíte zadat adresář se soubory této mapy, což v případě GSAKu bude datový adresář GSAKu, v případě GeoGetu tam dejte adresář pluginu (nebo klidně i celého datového adresáře), v případě jiného použití jistě sami víte :). Pak je třeba ještě znovu obnovit stránku a mělo by vše fungovat. To poznáte tak, že šipka vlevo na mapě je synchronizována s vaší pozicí ve Street View. Pokud ani toto nepomůže, tak se musíte smířit s omezeními nebo používat klasickou verzi 3, kde tato omezení nejsou.\n" +
    "\n" +
    "Je slušná šance, že v IE vám to poběží korektně i bez výše zmíněných změn. V Opeře by vám to mělo fungovat s použitím postupu výše. Ve Firefoxu a Chrome se mi to nepovedlo rozchodit vůbec, ale u vás to s použitím postupu výše fungovat může.\n",
  
  "English": "" +
    "Please copy text below to the clipboard, because it won't display again (at least until you update to a newer version).\n" +
    "\n" +
    "Now you are running Street View version 2, which is different version 3 in some areas - it's quicker, it can be switched to fullscreen, it doesn't show waypoints right within Street View window.\n" +
    "\n" +
    "But there are also significant problems, because this version is Flash based and Flash have some inherent problems when communicating from internet with local based JavaScript. These problems include not functioning synchronization of Street View with the map and also when you resize Street View, your position will change to last known position (that is - position when you activated Street View or changed its position by moving the arrow).\n" +
    "\n" +
    "You need at least Flash version 11.5 to attempt to resolve these problems (update if necessary).\n" +
    "\n" +
    "Right click on the Street View window or other Flash object within the browser. Then click on \"Global Settings\", then on \"Advanced\", then on \"Trusted Location Settings\". There you need to add path to the map. If you ran the map from GSAK, put entire GSAK data folder there. If you ran the map from GeoGet, put path to the plugin there (or entire GeoGet data folder). If neither, then you must figure out the path on your own :). After this you need to reload the page.\n" +
    "\n" +
    "If the green arrow on the map moves when you move around in Street View, everything works as intended. If not, you have to live with limitations (or use version 3, which doesn't have there problems). If you are using IE, chances are it works out of the box without any above mentioned changes.\n"
}

var StreetViewDiv = null;
var StreetViewJSLoaded = false;
var StreetViewMap = null;
var markerStreetViewLayer = null;
var StreetViewMarker = null;
var StreetViewMarkerDiv = null;
var svSizeButton;
var svCloseButton;
var lastRotation = 0;
var flashBlocked = true;
var flashBlockedLastCoords = null;
StreetViewInit();

function restartStreetView(coords, moveSV)
{
  if (coords)
  {
    StreetViewMarker.setCoords(new SMap.Coords.fromWGS84(coords.lng(), coords.lat()));
    
    if (moveSV)
    {
      StreetViewMap.afterMarkerDrag = true;
      try{StreetViewMap.setLocationAndPOV(coords);} catch(ex){};     
    }
    
    flashBlockedLastCoords = coords;
  }
  
  StreetViewMap.remove();
  
//   for (var i = 0; i < StreetViewDiv.childNodes.length; i++)
//     log(StreetViewDiv.childNodes[i].tagName + " / " + StreetViewDiv.childNodes[i].id)

  StreetViewDiv.innerHTML = "";
  createResizeButton();
  createCloseButton();
  
  StreetViewMap = new GStreetviewPanorama(StreetViewDiv);
  setStreetViewListeners();
}

function setStreetViewListeners()
{
  // change position - move marker
  GEvent.addListener(StreetViewMap, 'initialized', function(e)
  {
    flashBlocked = false;
    var coords = new SMap.Coords.fromWGS84(e.lng, e.lat);
    StreetViewMarker.setCoords(coords);
    m.setCenter(coords);
    
    // move back after error
    StreetViewMarker.lastCoords = e;
  });
  
  // error - move SV to previous position
  GEvent.addListener(StreetViewMap, 'error', function(e)
  {
    //if (StreetViewMarker.lastCoords)
    //  try{StreetViewMap.setLocationAndPOV(StreetViewMarker.lastCoords);} catch(ex){};   
  });    

  // change orientation - rotate marker
  GEvent.addListener(StreetViewMap, 'yawchanged', function(e)
  {
    // when POV set manually after marker drag, don't rotate marker on first call (because this function gets called twice)
    if (StreetViewMap.afterMarkerDrag)
    {
      StreetViewMap.afterMarkerDrag = false;
      return;
    }

    svRotateElement(StreetViewMarkerDiv, StreetViewMap.getPOV().yaw - 90);
  });
}

// create button to change size of SV from 50% to 100% or vice versa
function createResizeButton()
{
  svSizeButton = JAK.mel("div", {id: "sv_size_button", className: "button sidebar_button", innerHTML: settings.streetview_width});  
  StreetViewDiv.appendChild(svSizeButton);
  
  EventUtil.addHandler(svSizeButton, "click", function(e, target)
  {
    svResize(true, new SMap.Coords.fromEvent(e, m));
  });
}

// create close button
function createCloseButton()
{
  svCloseButton = JAK.mel("div", {id: "sv_close_button", className: ""});  
  StreetViewDiv.appendChild(svCloseButton);
  
  EventUtil.addHandler(svCloseButton, "click", function(e, target)
  {
    StreetViewDiv.style.display = "none";
    remove_class(document.documentElement, "streetview_on");
    markerStreetViewLayer.disable();
    mapadiv.style.display = "";
    mapadiv.style.width = "";
    createDefaultMapControls();
    m.syncPort();
  });
}

function LoadStreetView(coords)
{
  // on first start, get value from global variable
  if (window.startStreetViewCoords)
  {
    coords = window.startStreetViewCoords;
    window.startStreetViewCoords = null;
  }

  var latlon = coords.toWGS84("LatLon");
  var lat = latlon[0];
  var lon = latlon[1];

  if (!StreetViewJSLoaded)
  {
    StreetViewJSLoaded = true;
    StreetViewMap = new GStreetviewPanorama(StreetViewDiv);

    // add marker layer
    markerStreetViewLayer = new SMap.Layer.Marker();
    m.addLayer(markerStreetViewLayer);

    // add marker
    StreetViewMarkerDiv = JAK.mel("div", {className: "StreetViewMarker"});
    StreetViewMarker = new SMap.Marker(new SMap.Coords.fromWGS84(lon, lat), null, {anchor: {left: 17,top: 19}, url: StreetViewMarkerDiv});
    StreetViewMarker.decorate(SMap.Marker.Feature.Draggable);
    markerStreetViewLayer.addMarker(StreetViewMarker);
 
    // marker drag --> change position
    m.getSignals().addListener(window, "marker-drag-stop", function(e)
    {
      if (e.target !== StreetViewMarker)
        return;

      if (flashBlocked)
      {
        restartStreetView(CoordsToGoogle(e.target.getCoords()), true);
      }
      else
      {
        StreetViewMap.afterMarkerDrag = true;
        try{StreetViewMap.setLocationAndPOV(CoordsToGoogle(e.target.getCoords()), StreetViewMap.getPOV());} catch(ex){};      
      }
    });
    
    // marker left-click --> change heading
    EventUtil.addHandler(StreetViewMarkerDiv, "click", function(e, target)
    {
      // gecko doesn't work correctly
      if (JAK.Browser.client == "gecko")
        return;
        
      var width = target.offsetWidth;
      var height = target.offsetHeight;
      var x = e.layerX || /* IE7-8 */ e.offsetX;
      var y = e.layerY || /* IE7-8 */ e.offsetY;

      x = x - width / 2;
      y = y - height / 2;
      
      var azimuth = Math.atan(x / y) * 180 / Math.PI;
      
      if (y > 0 && x > 0)
        azimuth = 180 - azimuth;
      else if (y > 0 && x < 0)
        azimuth = 180 + Math.abs(azimuth);
      else if (y < 0 && x > 0)
        azimuth = Math.abs(azimuth);
      else if (y < 0 && x < 0)
        azimuth = 360 - azimuth;       

      StreetViewMap.setPOV({yaw: (azimuth) % 360, pitch: 0, zoom: 0});
    });
    
    // set listeners
    setStreetViewListeners();
    
    // create buttons
    createResizeButton();
    createCloseButton();

    // prevent PAGE UP, PAGE DOWN, ...
    EventUtil.addHandler(StreetViewDiv, 'keydown', function (e)
    {
      // not ENTER
      if (e.keyCode == 13)
        return;
    
      EventUtil.preventDefault(e);
      EventUtil.stopPropagation(e);
    }, true);
    
    // on window resize, adjust SV size
    EventUtil.addHandler(window, "resize", function()
    {
      if (!contains_class(document.documentElement, "streetview_on"))
        return;

      StreetViewDiv.style.height = mapadiv.offsetHeight - 2 /* 1px border */ + "px";
      svResize();  
    });
    
    // when not testing, prevent all errors
    if (!window.testing)
    {
      EventUtil.addHandler(window, "error", function(e)
      {
        //if (/maps\.gstatic\.com/.test(e.filename))
          return true;        
      });
    }
    
//     // display warning, when flash is blocked from accessing JS
//     window.setTimeout(function()
//     {
//       if (settings.flashStreetViewWarning || !flashBlocked)
//         return;
//     
//       ui("copybox", {text: tr("_flash_blocked_warning"), width: 600, height: 200});
//     }, 10000);
    
    if (!settings.flashStreetViewWarning)
    {
      ui("copybox", {text: tr("_flash_blocked_warning"), width: 600, height: 300});
    
      settings.flashStreetViewWarning = true;
    }
  }

  add_class(document.documentElement, "streetview_on");
  StreetViewDiv.style.display = "";
  try{StreetViewMap.setLocationAndPOV(CoordsToGoogle(coords));} catch(ex){}; 
  markerStreetViewLayer.enable();
  svRotateElement(StreetViewMarkerDiv, (StreetViewMap.getPOV().yaw || 0) - 90);

  svResize(false);
  flashBlockedLastCoords = CoordsToGoogle(coords);
}

// load after right click (not the first time, for that use LoadStreetView)
function reloadStreetView(coords)
{
  flashBlockedLastCoords = CoordsToGoogle(coords);
  add_class(document.documentElement, "streetview_on");
  StreetViewDiv.style.display = "";
  //try{StreetViewMap.setLocationAndPOV(CoordsToGoogle(coords));} catch(ex){}; 
  markerStreetViewLayer.enable();
  
  var pov = StreetViewMap.getPOV();
  svRotateElement(StreetViewMarkerDiv, (pov.yaw || 0) - 90);
  svResize(false);
  
  // always restart, because of unknown bug (in Opera, but probably elsewhere) where the position wouldn't update
  restartStreetView(flashBlockedLastCoords, true);
  StreetViewMap.afterMarkerDrag = true;
  try{StreetViewMap.setLocationAndPOV(flashBlockedLastCoords, pov);} catch(ex){};   
}

function svResize(advance)
{
  var sizes = {"25%": "50%", "50%": "75%", "75%": "100%", "100%": "25%"};
  var complement_size = {"25%": "75%", "50%": "50%", "75%": "25%", "100%": "0%"};
  
  var size = advance ? sizes[svSizeButton.innerHTML] : settings.streetview_width;
  var map_size = complement_size[size];
  
  // resize divs
  StreetViewDiv.style.width = size;
  mapadiv.style.width = map_size;
  mapadiv.style.display = (size == "100%") ? "none" : "";
  
  // set button to value 
  svSizeButton.innerHTML = size;
  settings.streetview_width = size;
  
  // synchronize google maps  
  StreetViewMap.checkResize();
  
  if (flashBlocked && flashBlockedLastCoords)
  {
    restartStreetView(flashBlockedLastCoords, true);
  }

  // synchronize mapy.cz (skip 100% to fix IE7-8 error)
  if (size !== "100%")
  {
    m.syncPort();
    createDefaultMapControls();
  }
}

function loadMaps() 
{
	google.load("maps", "2", {"callback" : LoadStreetView});
}

function StreetViewInit()
{
  if (!window.StreetViewDiv)
  {
    StreetViewDiv = document.createElement("div");
    StreetViewDiv.id = "StreetViewDiv";
    StreetViewDiv.style.display = "none";
    
    //loadJS("http://maps.google.com/maps?file=api&amp;v=2&amp;key=AIzaSyDiFOzfzt2mffXUcUZtaJ65KcMPfxk3yMk&sensor=false&callback=loadMaps");
    loadJS("http://www.google.com/jsapi?key=AIzaSyDiFOzfzt2mffXUcUZtaJ65KcMPfxk3yMk&callback=loadMaps");
  }

  mapadiv.parentNode.insertBefore(StreetViewDiv, mapadiv.nextSibling);
  StreetViewDiv.style.height = mapadiv.offsetHeight - 2 /* 1px border */ + "px";

  if (StreetViewMap)
  {
    StreetViewMap.checkResize();

    if (contains_class(document.documentElement, "streetview_on"))
      svResize(false);

    // IE fix
    if (JAK.Browser.client == "ie")
    {
      var coords = StreetViewMap.getLatLng();
      var pov = StreetViewMap.getPOV(); 

      restartStreetView();
      
      StreetViewMap.afterMarkerDrag = true;
      try{StreetViewMap.setLocationAndPOV(coords, pov);} catch(ex){};       
    }
  }
}

function CoordsToGoogle(coords)
{
  var latlon = coords.toWGS84("LatLon");
  var lat = lat || latlon[0];
  var lon = lon || latlon[1];
  return (new GLatLng(lat, lon));
}

function svRotateElement(elm, deg)
{
  lastRotation = deg;
  return(rotateElement(elm, deg));
}

function rotateElement(elm, deg)
{
  var degStr = "rotate(" + deg + "deg)";
  var properties = ["WebkitTransform", "MozTransform", "OTransform", "msTransform", "transform"];
  var p;
  
  while (p = properties.shift())
  {
    if (typeof elm.style[p] !== 'undefined')
      elm.style[p] = degStr;  
  }
  
  /* IE7-8 */
  var rad = deg * Math.PI * 2 / 360;
  costheta = Math.cos(rad);
  sintheta = Math.sin(rad);

  var a = parseFloat(costheta).toFixed(8);
  var c = parseFloat(-sintheta).toFixed(8);
  var b = parseFloat(sintheta).toFixed(8);
  var d = parseFloat(costheta).toFixed(8);

  if (typeof elm.style.msTransform === 'undefined' && typeof elm.style.transform === 'undefined')
    elm.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11=" + a + ", M12=" + c + ", M21=" + b + ", M22=" + d + ",sizingMethod='auto expand')";
}
